//----------------------------------------------------------------------------
//~Author: Fabio Arciniegas A.
//~Name: URLSaver
//~Description: This class saves the contents of a given URL to a local disk
//
// Version $Id$
//~Important: See distribution note at the end of this file.

//----------------------------------------------------------------------------
//             This code has been formated to fit in 80 columns
//----------------------------------------------------------------------------
package com.faa.dom.tutorial;


import java.net.  URL;
import java.io.   File;
import java.io.   FileOutputStream;
import java.io.   InputStream;
import java.lang. Thread;

import java.net. MalformedURLException;
import java.io.  IOException;



  /** 
      This class saves the contents of a given URL to a local disk
   */
public class URLSaver extends Thread
{

// ******************************************         Constructors / Destructors
/**
 * A Constructor that sets the base URL and Base directories.
 * @param baseURL a base URL String for subsequent calls to saveURLAs. Note that
    this doesn't mean the URLSaver cannot be used to save URLs outside the base.
    It is just a matter of convenience.
 * @param baseDir a base Directory name for subsequent calls to saveURLAs
 */

    public URLSaver(String newBaseURL, String newBaseDir)
           throws MalformedURLException
    {
	try
	{
	    baseURL = new URL(newBaseURL);
	    baseDir = newBaseDir == null ? "" : newBaseDir;
	}
	catch(MalformedURLException e)
	{
	    System.out.println("Bad base URL for a URLSaver.");
	    throw e;
	}
    }

    
// ******************************************                        Persistence

    /** 
	Save the given URL to a file, using a particular filename.
	Both the filename and the URL can be relative, in which case,
	the previously set bases will be taken into consideration.
      @param url the source URL
      @param filename the target filename
      @exception MalformedURLException if the URL is not valid
      @exception IOException if the file is not writable
   */
 
    public void saveURLAs(String url,String filename) 
	throws MalformedURLException, IOException
    {
	File f = new File(new File(baseDir) , filename);
	
	try
	{
	    target = new FileOutputStream(f);
	}
	catch(IOException e)
	{
	    System.out.println("Unwrittable file " + filename);
	    throw e;
	}
	try
	{
	    if(url.startsWith("http://")) //it is absolute
		source = new URL(url);
	    else
		source = new URL(baseURL,url);
	}
	catch(MalformedURLException e)
	{
	    System.out.println("Bad base URL for a URLSaver.");
	    throw e;
	}
	// All is well. Source and target are ready. Let's begin downloading
	start();
    }

  /** 
      save url using the name of the file on the url and the default directory
      @param String url the target url
      @exception MalformedURLException if the URL is not valid
      @exception IOException if the file is not writable
   */
    public void saveURL(String url)
	throws MalformedURLException, IOException
    {
	URL u = new URL(baseURL,url);
	saveURLAs(url,new File(u.getFile()).getName());
    }
    
    
// ******************************************                             Thread
    public void run()
    {
	try
	{
	    InputStream s = source.openStream();
	    int c;
	    while((c = s.read()) != -1)
		target.write(c);
	    target.close();
	}
	catch(IOException e)
	{
	    System.err.println("Error Downloading " + source.toString());
	    System.err.println(e);
	}
    }
    

// ******************************************                          Atributes

    private URL    baseURL;
    private String baseDir;    
    private URL    source;
    private FileOutputStream target;
}

